from django.conf import settings
from django.http import HttpResponse, HttpResponseNotFound
from django.views.generic import View

from ember_index.utils import path_for, replace_base_url


class IndexView(View):
    '''A class based view for serving index files of Ember applications.

    Every application has a `current` index file and one or many revisioned index files.

    These index files are accessible by `GET` request on this view.

    Attributes:
        http_method_names (list): A list of supported HTTP methods.
        manifest (string): A name of Ember application.
        adapter (object): An adapter for index provider.
        regex (string): A regex from Ember application is served.
        request: (django.http.HttpRequest): An instance of the current request.

    '''

    http_method_names = ['get', 'head', 'options']

    manifest = None
    adapter = None
    regex = None

    request = None

    def get(self, request, revision='current'):
        '''Handle `GET` request for index of Ember application.

        Keyword arguments:
            request (django.http.HttpRequest): An instance of HTTP request.
            revision (string): A requested revision, which defaults to `current`.

        Returns:
            An instance of `django.http.HttpResponse` with index as it's content.

            If no index is found then an instance of `django.http.HttpResponseNotFound` is returned.

        '''
        self.request = request

        index_key = self.index_key(revision)
        index = self.adapter.index_for(index_key)

        if not index:
            return self.index_not_found(revision)

        index = self.process_index(index, revision)
        return self.index_response(index)

    def head(self, request, revision='current'):
        '''Handle `HEAD` request for index of Ember application.

        Keyword arguments:
            request (django.http.HttpRequest): An instance of HTTP request.
            revision (string): A requested revision, which defaults to `current`.

        Returns:
            A same response like `GET` method, but without any content.

        '''
        response = self.get(request, revision)
        response.content = b''
        return response

    def index_key(self, revision):
        '''Join manifest and revision with `:` to build an index key.

        Keyword arguments:
            revision (string): A requested revision.

        Returns:
            A key used to index lookup.

        '''
        return ':'.join([self.manifest, revision])

    def index_not_found(self, revision):
        '''Return `django.http.HttpResponseNotFound`.

        Keyword arguments:
            revision (string): A requested revision.

        Returns:
            An instance of `django.http.HttpResponseNotFound` class.

        '''
        content = 'No index found for revision `{1}` of the manifest `{0}`.'
        return HttpResponseNotFound(content.format(self.manifest, revision))

    def index_response(self, index):
        '''Return index within `django.http.HttpResponse`.

        Keyword arguments:
            index (string): An index for requested revision.

        Returns:
            An instance of `django.http.HttpResponse` class with index as it's content.

        '''
        return HttpResponse(index)

    def process_index(self, index, revision):
        '''A hook which allows to modify an index.

        By default it replaces `baseURL` in environment configuration.

        If CSRF protection is enabled, then `meta` tag with CSRF token will be provided.

        Keyword arguments:
            index (string): An index for requested revision.
            revision (string): A requested revision.

        Returns:
            A processed index.

        '''
        index = replace_base_url(index, revision, self.path)

        if self.is_csrf_protection_enabled:
            index = self.append_meta_with_csrf_token(index)

        return index

    @property
    def path(self):
        return path_for(self.regex)

    @property
    def is_csrf_protection_enabled(self):
        return 'django.middleware.csrf.CsrfViewMiddleware' in settings.MIDDLEWARE_CLASSES

    @property
    def csrf_meta_name(self):
        return 'X-CSRFToken'  # HTTP_X_CSRFTOKEN

    def append_meta_with_csrf_token(self, index):
        '''Return index with CSRF token in meta tag named `X-CSRFToken`.

        Keyword arguments:
            index (string): An index for requested revision.

        Returns:
            An index with CSRF token.

        '''
        from django.middleware.csrf import get_token

        start = index.index('</head>')
        meta = '<meta name="{0}" content="{1}">'
        meta = meta.format(self.csrf_meta_name, get_token(self.request))

        return index[:start] + meta + index[start:]



def replace_base_url(index, revision, path):
    '''Replace base URL at environment configuration
       and optionally at base tag in index.

    Keyword arguments:
        index (string): An index for requested revision.
        revision (string): An requested revision.
        path (string): A path from Ember application is served.

    Returns:
        An index with replaced `baseURL`.

    Raises:
        ValueError: When `baseURL` can not be found in index.

    '''
    base_url = base_url_for(path, revision)

    index = replace_base_url_in_base_tag(index, base_url)
    index = replace_base_url_in_meta_tag(index, base_url)

    return index


def replace_base_url_in_base_tag(index, base_url):
    '''Replace base URL at base tag if present.

    Keyword arguments:
        index (string): An index for requested revision.
        base_url (string): A base URL to replace.

    Returns:
        An index with replaced `href` attribute at `base` tag`.

    '''
    base = '<base href="'
    start = index.find(base)

    if start is -1:
        return index

    start += len(base)
    end = index.index('"', start)

    return index[:start] + base_url + index[end:]


def replace_base_url_in_meta_tag(index, base_url):
    '''Replace base URL at environment configuration in index.

    Keyword arguments:
        index (string): An index for requested revision.
        base_url (string): A base URL to replace.

    Returns:
        An index with replaced `baseURL` at environment configuration.

    Raises:
        ValueError: When `baseURL` can not be found in index.

    '''
    base = 'baseURL%22%3A%22'
    start = index.index(base) + len(base)
    end = index.index('%22', start)

    return index[:start] + base_url + index[end:]


def base_url_for(path, revision):
    '''Create a base URL for path and requested revision.

    Keyword arguments:
        path (string): A path from Ember application is served.
        revision (string): A requested revision.

    Returns:
        A base URL.

    '''
    if revision == 'current':
        return path
    return '{0}r/{1}/'.format(path, revision)


def path_for(regex):
    '''Create path by replacing '^' with '/' in regex.

    Keyword arguments:
        regex (string): A regex from Ember application is served.

    Returns:
        A path from Ember application is served.

    '''
    return regex.replace('^', '/')



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'DESCRIPTION.rst')) as description:
    LONG_DESCRIPTION = description.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-ember-index',
    version='0.7.0',
    packages=['ember_index'],
    include_package_data=True,
    test_suite='tests.runner.run_tests',
    license='MIT License',
    description='A Django app to serve Ember index files.',
    long_description=LONG_DESCRIPTION,
    url='http://bobisjan.com/django-ember-index',
    author='Jan Bobisud',
    author_email='me@bobisjan.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.4',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Text Processing',
        'Topic :: Text Processing :: Markup',
        'Topic :: Text Processing :: Markup :: HTML',
    ],
)
